/*

	Sender and Reciever are simple AppleEvent programs that demonstrate
	all the permutations of interaction levels for sending and recieving
	AppleEvents.  Have fun with them.
	
		C.K. Haun
		Apple DTS
	
	
	This application recieves the simple event.

		
		08/24/98	afb		Updated to Universal Interfaces 3.x, CWPro 3
							Much cleanup.
							Fixed bug in idleProc() where global event record was being used
							rather than the event record passed to the idle function.
							Move the global event record to DoEvent().
							Moved much of the code from main() to helper functions to help clarify things.
							Added pascal qualifier to Apple event handler declarations and definitions.
*/


//**************	Universal Headers		********************************

#include <AppleEvents.h>
#include <Devices.h>
#include <Dialogs.h>
#include <DiskInit.h>
#include <EPPC.h>
#include <Errors.h>
#include <Events.h>
#include <Fonts.h>
#include <Gestalt.h>
#include <Memory.h>
#include <Menus.h>
#include <OSUtils.h>
#include <PPCToolbox.h> 
#include <Packages.h>
#include <Processes.h>
#include <Quickdraw.h>
#include <Resources.h>
#include <ToolUtils.h>
#include <Types.h>
#include <Windows.h>


//**************	Prototypes				********************************

		void	InitToolbox( void );
		void	SetupMenus( void );
		void	DoEvent( EventRecord *theEventPtr );
		
		void	DoDiskEvents( long dinfo );			/* hi word is error code, lo word is drive number */
		void	DrawMain( WindowPtr drawIt );
		void	DoSelected( long val );
		OSErr	InitAEStuff( void );
		void	DoHighLevel(EventRecord *AERecord);
		void	DoDaCall(MenuHandle themenu, long theit);
		
pascal	OSErr	AEOpenHandler(AppleEvent *messagein, AppleEvent *reply, long refIn);
pascal	OSErr	AEOpenDocHandler(AppleEvent *messagein, AppleEvent *reply, long refIn);
pascal	OSErr	AEPrintHandler(AppleEvent *messagein, AppleEvent *reply, long refIn);
pascal	OSErr	AEQuitHandler(AppleEvent *messagein, AppleEvent *reply, long refIn);
pascal	OSErr	AESimpleHandler(AppleEvent *messagein, AppleEvent *reply, long refIn);
		
pascal	Boolean idleProc(EventRecord *eventIn, long *sleep, RgnHandle *mouseRgn);
		
		
//**************		Definitions			********************************

//	Interaction menu items

#define kSelfInteractItem 1
#define kLocalInteractItem 2
#define kAllInteractItem 3

short gInteractArray[4] = 
{
	nil, kAEInteractWithSelf, kAEInteractWithLocal, kAEInteractWithAll
};


#define kMBarID		128
#define kAppleMenu	128
#define kFileMenu	129
#define kEditMenu	130
#define kToolsMenu	131


//**************		Globals				********************************

Boolean		gQuit,
			gInBackground;
			

MenuHandle	gAppleMenuHandle,
			gFileMenuHandle,
			gEditMenuHandle,
			gToolMenuHandle;
			
Handle		gMymenu;				/* my menu bar handle */

//AEAddressDesc	targetAddress;		/* address of the person to get the data from */


#define kSimpleEvent 'SIMP'
#define kSimpleClass 'Simp'



WindowPtr	myWindow;
short		gInteractNow = kSelfInteractItem;
AEIdleUPP	gAEIdleUPP;

#if defined( powerc )  &&  !defined( __MWERKS__ )
	QDGlobals	qd;
#endif

//**************************************************************************

void main( void )
{
	EventRecord		theEvent;
	OSErr			anErr;
	
	InitToolbox();
	
	anErr = InitAEStuff();
	if ( anErr != noErr )
	{
		ExitToShell();
	}
	
	SetupMenus();
	GetNewWindow(128, nil, (WindowPtr)-1);
	
	do {
	
		WaitNextEvent(everyEvent, &theEvent, 30, nil);
		
		DoEvent( &theEvent );
		
	} while (gQuit != true);
	
	return;
}

//**************************************************************************

void InitToolbox( void )
{
	MaxApplZone();
	InitGraf((Ptr)&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();
	
	return;
}//end InitToolbox

//**************************************************************************

void SetupMenus( void )
{
	gMymenu = GetNewMBar(kMBarID);
	SetMenuBar(gMymenu);
	
	gAppleMenuHandle = GetMenuHandle(kAppleMenu);
	gFileMenuHandle = GetMenuHandle(kFileMenu);
	gEditMenuHandle = GetMenuHandle(kEditMenu);
	gToolMenuHandle = GetMenuHandle(kToolsMenu);
	
	AppendResMenu(gAppleMenuHandle, 'DRVR');
	DrawMenuBar();
	
	CheckItem(gToolMenuHandle, gInteractNow, true);
	
	return;
}//end SetupMenus

//**************************************************************************

void DoEvent( EventRecord *theEventPtr )
{
	WindowPtr		twindow;
	
	switch (theEventPtr->what)
	{
		case nullEvent:
			/* no nul processing in this sample */
			break;
			
		case updateEvt:
			DrawMain((WindowPtr)theEventPtr->message);		/* draw whatever window needs an update */
			break;
			
		case mouseDown:
			/* first see where the hit was */
			switch (FindWindow(theEventPtr->where, &twindow))
			{
				case inDesk:							/* if they hit in desk, then the process manager */
					break;
													/* will switch us out, we don't need to do anything */
				case inMenuBar:
					DoSelected(MenuSelect(theEventPtr->where));
					break;
					
				case inSysWindow:
					/* pass to the system */
					SystemClick(theEventPtr, twindow);
					break;
				case inContent:
					break;
					
				case inDrag:
					if (twindow == FrontWindow())
						DragWindow(twindow, theEventPtr->where, &qd.screenBits.bounds);
					break;
					
				case inGrow:
				case inGoAway:
					/* don't care */
					break;
				default:
					break;
			}
		case mouseUp:
			/* don't care */
			break;
			
			/* same action for key or auto key */
		case keyDown:
		case autoKey:
			if (theEventPtr->modifiers & cmdKey)
				DoSelected(MenuKey(theEventPtr->message & charCodeMask));
			break;
			
		case keyUp:
			/* don't care */
			break;
			
		case diskEvt:
			/* I don't do anything special for disk events, this just passes them */
			/* to a function that checks for an error on the mount */
			DoDiskEvents(theEventPtr->message);
			break;
			
		case activateEvt:
			/* Draws on a window activate.	Other activate/deactivate stuff is */
			/* handled in either the ChangePlane function (for normal shuffling ) */
			/* or in the suspend/resume handler for layer swaps */
			if (theEventPtr->modifiers & activeFlag)
				DrawMain((WindowPtr)theEventPtr->message);
			break;
			
		case networkEvt:
			/* don't care */
			break;
			
		case driverEvt:
			/* don't care */
			break;
			
		case osEvt:		//	suspend/resume is also an activate/deactivate
			if ( ((theEventPtr->message >> 24) & 0x0FF) == suspendResumeMessage )	/* high byte of message */
			{
				gInBackground = ( (theEventPtr->message & resumeFlag) == 0 );
			}
			break;

			break;
			/* This dispatches high level events (AppleEvents, for example) */
			/* to our dispatch routine.	 This is NEW in the event loop for */
			/* System 7 */
		case kHighLevelEvent:
			DoHighLevel(theEventPtr);
			break;
			
		default:
			break;
	}
	
	return;
}//end DoEvent

//**************************************************************************
/*
	DoDaCall opens the requested DA.	 It's here as a seperate routine if you'd
	like to perform some action or just know when a DA is opened in your
	layer.  Can be handy to track memory problems when a DA is opened
	with an Option-open
*/

void DoDaCall(MenuHandle themenu, long theit)
{
	long qq;
	unsigned char DAname[255];
	
	GetMenuItemText(themenu, theit, DAname);
	qq = OpenDeskAcc(DAname);
}//end DoDaCall


//**************************************************************************
/*
	DoDiskEvents just checks the error code from the disk mount, 
	and puts up the 'Format' dialog (through DIBadMount) if need be
	You can do much more here if you care about what disks are 
	in the drive
*/

void DoDiskEvents(long dinfo)								/* hi word is error code, lo word is drive number */
{
	short hival, loval, tommy;
	Point fredpoint =  {
		40, 40
	};
	hival = HiWord(dinfo);
	loval = LoWord(dinfo);
	if (hival != noErr)										/* something happened */ {
		tommy = DIBadMount(fredpoint, dinfo);
	}
}//end DoDiskEvents

//**************************************************************************

void DrawMain(WindowPtr drawIt)
{
	BeginUpdate(drawIt);
	SetPort(drawIt);
	MoveTo(40, 70);
	TextFace(bold);
	TextSize(20);
	DrawString("\pThis Does Nothing");
	EndUpdate(drawIt);
	
	return;
}//end DrawMain

//**************************************************************************

void DoSelected(long val)
{
	short loval, hival;
	Boolean temp = false;
	loval = LoWord(val);
	hival = HiWord(val);
	
	switch (hival) {										/* switch off the menu number selected */
		case kAppleMenu:									/* Apple menu */
			if (loval != 1) {								/* if this was not About, it's a DA */
				DoDaCall(gAppleMenuHandle, loval);
			} else {
				Alert(128, nil);							/* do about box */
			}
			break;
		case kFileMenu:										/* File menu */
			gQuit = true;									/* only edit item */
			break;
		case kEditMenu:
			/* edit menu junk */
			/* don't care */
			break;
		case kToolsMenu:
			if (loval != gInteractNow) {
				CheckItem(gToolMenuHandle, gInteractNow, false);
				CheckItem(gToolMenuHandle, loval, true);
				AESetInteractionAllowed(gInteractArray[loval]);
				gInteractNow = loval;
			}
	}
	HiliteMenu(0);
	
	return;
}//end DoSelected

//**************************************************************************

OSErr InitAEStuff(void)
{
	OSErr		aevtErr = noErr;
	long		aLong = 0;
	Boolean		gHasAppleEvents = false;
	
	// Check this machine for AppleEvents.	If they are not here (ie not 7.0) then we exit
	aevtErr = Gestalt( gestaltAppleEventsAttr, &aLong );
	
	/*
		The following series of calls installs all our Apple event handlers.
		These handlers are added to the application event handler list that 
		the Apple Event Manager maintains.  So, whenever an Apple event happens
		and we call AEProcessAppleEvent, the Apple Event Manager will check our
		list of handlers and dispatch to the handler if there is one.
	*/
	
	if ( aevtErr == noErr )
	{
		aevtErr =  AEInstallEventHandler( kCoreEventClass, kAEOpenApplication, 
										  NewAEEventHandlerProc(AEOpenHandler), 0, false );
		if ( aevtErr == noErr )
		{
			aevtErr =  AEInstallEventHandler( kCoreEventClass, kAEOpenDocuments, 
											  NewAEEventHandlerProc(AEOpenDocHandler), 0, false);
		}
		if ( aevtErr == noErr )
		{
			aevtErr = AEInstallEventHandler( kCoreEventClass, kAEPrintDocuments, 
											 NewAEEventHandlerProc(AEPrintHandler), 0, false );
		}
		if ( aevtErr == noErr )
		{
			aevtErr = AEInstallEventHandler( kCoreEventClass, kAEQuitApplication, 
											 NewAEEventHandlerProc(AEQuitHandler), 0, false);
		}
		if ( aevtErr == noErr )
		{
			aevtErr = AEInstallEventHandler( kSimpleClass, kSimpleEvent, 
											 NewAEEventHandlerProc(AESimpleHandler),0, false);
		}
		if ( aevtErr == noErr )
		{
			/*	Create a UPP for the AppleEvent idle proc	*/
			gAEIdleUPP = NewAEIdleProc(idleProc);
		}
	}
	
	return aevtErr;
	
}//end InitAEStuff

//**************************************************************************

void DoHighLevel( EventRecord *theEvent )
{
	AEProcessAppleEvent( theEvent );
	
	return;
}//end DoHighLevel



//**************************************************************************
/*
	This is the standard Open Application event.  You'll get this as one of the (if not the )
	first events in your application.  This is where an untitled document would be opened.
*/
pascal OSErr AEOpenHandler( AppleEvent *messagein, AppleEvent *reply, long refIn )
{
	#pragma unused ( messagein, reply, refIn )
	
	//	Do nothing since this app doen't deal with documents.
	return( noErr );
	
}//end AEOpenHandler


//**************************************************************************
/*
	Open Doc, opens our documents.  Remember, this can happen at application start AND 
	anytime else.  If your app is up and running and the user goes to the desktop, hilites one 
	of your files, and double-clicks or selects Open from the finder File menu this event 
	handler will get called. Which means you don't do any initialization of globals here, or 
	anything else except open then doc.	
	
	SO-- Do NOT assume that you are at app start time in this 
	routine, or bad things will surely happen to you.
*/

pascal OSErr AEOpenDocHandler( AppleEvent *messagein, AppleEvent *reply, long refIn )
{
	#pragma unused ( messagein, reply, refIn )
	
	//	Of course nothing happens here -- the app has no docs, so no odoc events should ever get here
	return( errAEEventNotHandled );
	
}//end AEOpenDocHandler

//**************************************************************************
/*
	No printing handler in yet, so ignore this.
	The operation is functionally identical to the ODOC event, with the additon 
	of calling your print routine.
*/

pascal OSErr AEPrintHandler(AppleEvent *messagein, AppleEvent *reply, long refIn)
{
	#pragma unused ( messagein, reply,refIn )

	//	Of course nothing happens here -- the app has no docs, so no odoc events should ever get here
	return( errAEEventNotHandled );
	
}//end AEPrintHandler

//**************************************************************************
/*
	Standard Quit event handler, to handle a Quit event from the Finder, for example.
	 DO NOT CALL ExitToShell HERE  or you will never have a happy life.
*/

pascal OSErr AEQuitHandler(AppleEvent *messagein, AppleEvent *reply, long refIn)
{
	#pragma unused ( messagein, reply, refIn )
	
	gQuit = true;
	
	return( noErr );
}//end AEQuitHandler

//**************************************************************************

pascal OSErr AESimpleHandler( AppleEvent *messagein, AppleEvent *reply, long refIn )
{
#pragma unused ( messagein, reply, refIn )

	OSErr myErr = AEInteractWithUser( kAEDefaultTimeout, nil, gAEIdleUPP );
	
	if ( myErr != errAENoUserInteraction )
	{
		Alert( 444, nil );
	}
	
	return( noErr );
}//end AESimpleHandler

//**************************************************************************

pascal Boolean idleProc( EventRecord *eventIn, long *sleep, RgnHandle *mouseRgn )
{
	*sleep = 30;		//	This app doen't do much, so sleep most of the time.
	mouseRgn = nil;
	
	switch ( eventIn->what )
	{
		case nullEvent:
			/* no nul processing in this sample */
			break;
			
		case updateEvt:
		case activateEvt:
			DrawMain( (WindowPtr)eventIn->message );			/* draw whatever window needs an update */
			break;
			
		case osEvt:		//	suspend/resume is also an activate/deactivate
			if ( ((eventIn->message >> 24) & 0x0FF) == suspendResumeMessage )
			{
				gInBackground = ( (eventIn->message & resumeFlag) == 0 );
			}
			break;
	}
	
	return( noErr );	/* I'll wait forever */
}//end idleProc
